% Fits a data set from Scenario 3 using the partial, pseudo, full and
% no observation error ML methods. Likelihoods are optimizied 
% using fminsearch.
% Inputs to the function:
%       y : data matrix where each row is a replicate series
%       x : vector of true (simulated) population abundance 
%       pars : vector of true parameter values (used for initialisation)
%       nInit : number of starting values for optimization
% Output returned:
%       pbest1 : partial ML estimates
%       pbest2 : pseudo ML estimates 
%       pbest3 : full ML estimates
%       pbest4 : no observation error ML estimates
%
% Written by Jonas Knape (jonas.knape@slu.se).

function [pbest1 pbest2 pbest3 pbest4 minnLL] = fitModel(y,x,pars,nInit)
[d T]=size(y);        
a=pars(1);
c=pars(2);
s2=pars(3)^2;
s2o=pars(4)^2;
s2y=pars(5)^2;

options=optimset(optimset('fminsearch'),'MaxIter',5000,'MaxFunEvals',10000,'Display','Off','TolX',1e-8,'TolFun',1e-8);

% Generate random starting values for numerical maximisation
p0=[a c log(s2) log(s2y) log(s2o)];
p0=[p0;2+randn(nInit,1) 2*rand(nInit,1)-1 log(0.3*rand(nInit,1)) log(0.3*rand(nInit,1)) log(0.3*rand(nInit,1))];


yhat=mean(y,1);
bhat=mean(y,2)-mean(yhat);
S2=sum(var(y-bhat*ones(1,T)));
SE2=S2/(d*(T-1));

% Likelihood for partial ML
nLL1=@(p)(-kalman(yhat,exp(p(4)),p(2),p(1),exp(p(3)),yhat(1),10));
% Likelihood for pseudo ML
nLL2=@(p)(-kalman(yhat,SE2+exp(p(4)),p(2),p(1),exp(p(3)),yhat(1),10));
% Likelihood for full ML
nLL3=@(p)(fullNLL(yhat,p(1),p(2),exp(p(3)),exp(p(4)),exp(p(5)),yhat(1),10,d,S2));
% Likelihood for model without obsrevation error
nLL4=@(p)(-kalman(x,0,p(2),p(1),exp(p(3)),mean(y(:,1)),10));

minnLL=Inf*ones(4,1);
nLLval=Inf*ones(4,1);
pbest1=[NaN NaN NaN NaN];
pbest2=[NaN NaN NaN NaN];
pbest3=[NaN NaN NaN NaN];
pbest4=[NaN NaN NaN];

% Loop over starting values
for (i = 1:nInit)

    % Partial ML
    try
        exitFlag=0;
        p1=p0(i,1:4);
        [p1 v exitFlag] = fminsearch(nLL1,p1,options);
        if (exitFlag==1)
            nLLval(1)=v;
        else
            p1=[NaN NaN NaN NaN];
            nLLval(1)=Inf;
            warning('Partial ML optimization did not converge');
        end
    catch me
        p1=[NaN NaN NaN NaN];
        nLLval(1)=Inf;
   end
    if (minnLL(1)==Inf || nLLval(1)<minnLL(1))
        minnLL(1)=nLLval(1);
        pbest1=p1;
    end
    
    % Pseudo ML
    try
        exitFlag=0;
        p2=p0(i,[1:3 5]);
        [p2 v exitFlag] = fminsearch(nLL2,p2,options);
        if (exitFlag==1)
            nLLval(2)=v;
        else
            p2=[NaN NaN NaN NaN];
            nLLval(2)=Inf;
            warning('Pseudo ML optimization did not converge');
        end
    catch me
        p2=[NaN NaN NaN NaN];
        nLLval(2)=Inf;
    end
    if (minnLL(2)==Inf || nLLval(2)<minnLL(2))
        minnLL(2)=nLLval(2);
        pbest2=p2;
    end    

    % Full ML
    try
        exitFlag=0;
        p3=[p0(i,:)];
        [p3 v exitFlag] = fminsearch(nLL3,p3,options);
        if (exitFlag==1)
            nLLval(3)=v;
        else
            p3=NaN*ones(1,5+d-1);
            nLLval(3)=Inf;
            warning('Full ML optimization did not converge');
        end
    catch me
        p3=NaN*ones(1,5+d-1);
        nLLval(3)=Inf;
    end
    if (minnLL(3)==Inf || nLLval(3)<minnLL(3))
        minnLL(3)=nLLval(3);
        pbest3=p3;
    end

    
    % No observation error
    try
        exitFlag=0;
        p4=p0(i,1:3);
        [p4 v exitFlag] = fminsearch(nLL4,p4,options);
        if (exitFlag==1)
            nLLval(4)=v;
        else
            p4=[NaN NaN NaN];
            nLLval(4)=Inf;
            warning('No observation error ML did not converge');
        end
    catch me
        p4=[NaN NaN NaN];
        nLLval(4)=Inf;
    end
    if (minnLL(4)==Inf || nLLval(4)<minnLL(4))
        minnLL(4)=nLLval(4);
        pbest4=p4;
    end
    
end

pbest1(3:4)=exp(pbest1(3:4)/2);
pbest2(3:4)=exp(pbest2(3:4)/2);
pbest3(3:5)=exp(pbest3(3:5)/2);
pbest3=[pbest3 bhat'];
pbest4(3)=exp(pbest4(3)/2);
end
